@@ -7,8 +7,10 @@
 from __future__ import division
 
 import base64
+import collections
 import errno
 import functools
+import glob
 import os
 import re
 import socket
@@ -25,6 +27,7 @@ from . import _psutil_linux as cext
 from . import _psutil_posix as cext_posix
 from ._common import isfile_strict
 from ._common import memoize
+from ._common import memoize_when_activated
 from ._common import parse_environ_block
 from ._common import NIC_DUPLEX_FULL
 from ._common import NIC_DUPLEX_HALF
@@ -59,9 +62,11 @@ __extra__all__ = [
 # --- constants
 # =====================================================================
 
+POWER_SUPPLY_PATH = "/sys/class/power_supply"
 
 HAS_SMAPS = os.path.exists('/proc/%s/smaps' % os.getpid())
 HAS_PRLIMIT = hasattr(cext, "linux_prlimit")
+_DEFAULT = object()
 
 # RLIMIT_* constants, not guaranteed to be present on all kernels
 if HAS_PRLIMIT:
@@ -132,6 +137,8 @@ TCP_STATUSES = {
     "0B": _common.CONN_CLOSING
 }
 
+_DEFAULT = object()
+
 
 # =====================================================================
 # -- exceptions
@@ -275,6 +282,19 @@ def set_scputimes_ntuple(procfs_path):
     return scputimes
 
 
+def cat(fname, fallback=_DEFAULT, binary=True):
+    """Return file content."""
+    try:
+        f = open_binary(fname) if binary else open_text(fname)
+    except IOError:
+        if fallback != _DEFAULT:
+            return fallback
+        raise
+    else:
+        with f:
+            return f.read().strip()
+
+
 try:
     scputimes = set_scputimes_ntuple("/proc")
 except Exception:
@@ -606,6 +626,26 @@ def cpu_stats():
         ctx_switches, interrupts, soft_interrupts, syscalls)
 
 
+if os.path.exists("/sys/devices/system/cpu/cpufreq"):
+
+    def cpu_freq():
+        # scaling_* files seem preferable to cpuinfo_*, see:
+        # http://unix.stackexchange.com/a/87537/168884
+        ret = []
+        ls = glob.glob("/sys/devices/system/cpu/cpufreq/policy*")
+        # Sort the list so that '10' comes after '2'. This should
+        # ensure the CPU order is consistent with other CPU functions
+        # having a 'percpu' argument and returning results for multiple
+        # CPUs (cpu_times(), cpu_percent(), cpu_times_percent()).
+        ls.sort(key=lambda x: int(os.path.basename(x)[6:]))
+        for path in ls:
+            curr = int(cat(os.path.join(path, "scaling_cur_freq"))) / 1000
+            max_ = int(cat(os.path.join(path, "scaling_max_freq"))) / 1000
+            min_ = int(cat(os.path.join(path, "scaling_min_freq"))) / 1000
+            ret.append(_common.scpufreq(curr, min_, max_))
+        return ret
+
+
 # =====================================================================
 # --- network
 # =====================================================================
@@ -1022,6 +1062,68 @@ def disk_partitions(all=False):
     return retlist
 
 
+# =====================================================================
+# --- sensors
+# =====================================================================
+
+
+def sensors_temperatures():
+    """Return hardware (CPU and others) temperatures as a dict
+    including hardware name, label, current, max and critical
+    temperatures.
+
+    Implementation notes:
+    - /sys/class/hwmon looks like the most recent interface to
+      retrieve this info, and this implementation relies on it
+      only (old distros will probably use something else)
+    - lm-sensors on Ubuntu 16.04 relies on /sys/class/hwmon
+    - /sys/class/thermal/thermal_zone* is another one but it's more
+      difficult to parse
+    """
+    ret = collections.defaultdict(list)
+    # Will return an empty dict if path does not exist.
+    basenames = sorted(set(
+        [x.split('_')[0] for x in
+         glob.glob('/sys/class/hwmon/hwmon*/temp*_*')]))
+    for base in basenames:
+        unit_name = cat(os.path.join(os.path.dirname(base), 'name'))
+        label = cat(base + '_label', fallback='')
+        current = float(cat(base + '_input')) / 1000.0
+        high = cat(base + '_max', fallback=None)
+        critical = cat(base + '_crit', fallback=None)
+
+        if high is not None:
+            high = float(high) / 1000.0
+        if critical is not None:
+            critical = float(critical) / 1000.0
+
+        ret[unit_name].append((label, current, high, critical))
+
+    return ret
+
+
+def sensors_battery():
+    root = os.path.join(POWER_SUPPLY_PATH, "BAT0")
+    if not os.path.exists(root):
+        return None
+
+    power_plugged = cat(os.path.join(POWER_SUPPLY_PATH, "AC0/online"),
+                        fallback=b"0") == b"1"
+    energy_now = int(cat(root + "/energy_now"))
+    power_now = int(cat(root + "/power_now"))
+    percent = int(cat(root + "/capacity"))
+
+    if power_plugged:
+        secsleft = _common.POWER_TIME_UNLIMITED
+    else:
+        try:
+            secsleft = int(energy_now / power_now * 3600)
+        except ZeroDivisionError:
+            secsleft = _common.POWER_TIME_UNKNOWN
+
+    return _common.sbattery(percent, secsleft, power_plugged)
+
+
 # =====================================================================
 # --- other system functions
 # =====================================================================
@@ -1069,8 +1171,32 @@ def pids():
 
 
 def pid_exists(pid):
-    """Check For the existence of a unix pid."""
-    return _psposix.pid_exists(pid)
+    """Check for the existence of a unix PID."""
+    if not _psposix.pid_exists(pid):
+        return False
+    else:
+        # Linux's apparently does not distinguish between PIDs and TIDs
+        # (thread IDs).
+        # listdir("/proc") won't show any TID (only PIDs) but
+        # os.stat("/proc/{tid}") will succeed if {tid} exists.
+        # os.kill() can also be passed a TID. This is quite confusing.
+        # In here we want to enforce this distinction and support PIDs
+        # only, see:
+        # https://github.com/giampaolo/psutil/issues/687
+        try:
+            # Note: already checked that this is faster than using a
+            # regular expr. Also (a lot) faster than doing
+            # 'return pid in pids()'
+            with open_binary("%s/%s/status" % (get_procfs_path(), pid)) as f:
+                for line in f:
+                    if line.startswith(b"Tgid:"):
+                        tgid = int(line.split()[1])
+                        # If tgid and pid are the same then we're
+                        # dealing with a process PID.
+                        return tgid == pid
+                raise ValueError("'Tgid' line not found")
+        except (EnvironmentError, ValueError):
+            return pid in pids()
 
 
 def wrap_exceptions(fun):
@@ -1104,12 +1230,16 @@ class Process(object):
         self._ppid = None
         self._procfs_path = get_procfs_path()
 
+    @memoize_when_activated
     def _parse_stat_file(self):
         """Parse /proc/{pid}/stat file. Return a list of fields where
         process name is in position 0.
         Using "man proc" as a reference: where "man proc" refers to
-        position N, always subscract 2 (e.g starttime pos 22 in
+        position N, always substract 2 (e.g starttime pos 22 in
         'man proc' == pos 20 in the list returned here).
+
+        The return value is cached in case oneshot() ctx manager is
+        in use.
         """
         with open_binary("%s/%s/stat" % (self._procfs_path, self.pid)) as f:
             data = f.read()
@@ -1121,15 +1251,32 @@ class Process(object):
         fields_after_name = data[rpar + 2:].split()
         return [name] + fields_after_name
 
+    @memoize_when_activated
     def _read_status_file(self):
+        """Read /proc/{pid}/stat file and return its content.
+
+        The return value is cached in case oneshot() ctx manager is
+        in use.
+        """
         with open_binary("%s/%s/status" % (self._procfs_path, self.pid)) as f:
             return f.read()
 
+    @memoize_when_activated
     def _read_smaps_file(self):
         with open_binary("%s/%s/smaps" % (self._procfs_path, self.pid),
                          buffering=BIGGER_FILE_BUFFERING) as f:
             return f.read().strip()
 
+    def oneshot_enter(self):
+        self._parse_stat_file.cache_activate()
+        self._read_status_file.cache_activate()
+        self._read_smaps_file.cache_activate()
+
+    def oneshot_exit(self):
+        self._parse_stat_file.cache_deactivate()
+        self._read_status_file.cache_deactivate()
+        self._read_smaps_file.cache_deactivate()
+
     @wrap_exceptions
     def name(self):
         name = self._parse_stat_file()[0]
@@ -1217,6 +1364,11 @@ class Process(object):
         children_stime = float(values[15]) / CLOCK_TICKS
         return _common.pcputimes(utime, stime, children_utime, children_stime)
 
+    @wrap_exceptions
+    def cpu_num(self):
+        """What CPU the process is on."""
+        return int(self._parse_stat_file()[37])
+
     @wrap_exceptions
     def wait(self, timeout=None):
         try:
@@ -1372,8 +1524,9 @@ class Process(object):
         if not ctxsw:
             raise NotImplementedError(
                 "'voluntary_ctxt_switches' and 'nonvoluntary_ctxt_switches'"
-                "lines were not found in /proc/%s/status; the kernel is "
-                "probably older than 2.6.23" % self.pid)
+                "lines were not found in %s/%s/status; the kernel is "
+                "probably older than 2.6.23" % (
+                    self._procfs_path, self.self.pid))
         else:
             return _common.pctxsw(int(ctxsw[0]), int(ctxsw[1]))
 
@@ -1433,18 +1586,33 @@ class Process(object):
     def cpu_affinity_get(self):
         return cext.proc_cpu_affinity_get(self.pid)
 
+    def _get_eligible_cpus(
+            self, _re=re.compile(b"Cpus_allowed_list:\t(\d+)-(\d+)")):
+        # See: https://github.com/giampaolo/psutil/issues/956
+        data = self._read_status_file()
+        match = _re.findall(data)
+        if match:
+            return list(range(int(match[0][0]), int(match[0][1]) + 1))
+        else:
+            return list(range(len(per_cpu_times())))
+
     @wrap_exceptions
     def cpu_affinity_set(self, cpus):
         try:
             cext.proc_cpu_affinity_set(self.pid, cpus)
         except (OSError, ValueError) as err:
             if isinstance(err, ValueError) or err.errno == errno.EINVAL:
-                allcpus = tuple(range(len(per_cpu_times())))
+                eligible_cpus = self._get_eligible_cpus()
+                all_cpus = tuple(range(len(per_cpu_times())))
                 for cpu in cpus:
-                    if cpu not in allcpus:
+                    if cpu not in all_cpus:
                         raise ValueError(
                             "invalid CPU number %r; choose between %s" % (
-                                cpu, allcpus))
+                                cpu, eligible_cpus))
+                    if cpu not in eligible_cpus:
+                        raise ValueError(
+                            "CPU number %r is not eligible; choose "
+                            "between %s" % (cpu, eligible_cpus))
             raise
 
     # only starting from kernel 2.6.13
