@@ -104,7 +104,7 @@ class TestProcess(unittest.TestCase):
         sig = p.wait()
         self.assertFalse(psutil.pid_exists(test_pid))
         if POSIX:
-            self.assertEqual(sig, signal.SIGKILL)
+            self.assertEqual(sig, -signal.SIGKILL)
 
     def test_terminate(self):
         sproc = get_test_subprocess()
@@ -114,7 +114,7 @@ class TestProcess(unittest.TestCase):
         sig = p.wait()
         self.assertFalse(psutil.pid_exists(test_pid))
         if POSIX:
-            self.assertEqual(sig, signal.SIGTERM)
+            self.assertEqual(sig, -signal.SIGTERM)
 
     def test_send_signal(self):
         sig = signal.SIGKILL if POSIX else signal.SIGTERM
@@ -124,7 +124,7 @@ class TestProcess(unittest.TestCase):
         exit_sig = p.wait()
         self.assertFalse(psutil.pid_exists(p.pid))
         if POSIX:
-            self.assertEqual(exit_sig, sig)
+            self.assertEqual(exit_sig, -sig)
             #
             sproc = get_test_subprocess()
             p = psutil.Process(sproc.pid)
@@ -155,7 +155,7 @@ class TestProcess(unittest.TestCase):
         p.kill()
         code = p.wait()
         if POSIX:
-            self.assertEqual(code, signal.SIGKILL)
+            self.assertEqual(code, -signal.SIGKILL)
         else:
             self.assertEqual(code, 0)
         self.assertFalse(p.is_running())
@@ -165,7 +165,7 @@ class TestProcess(unittest.TestCase):
         p.terminate()
         code = p.wait()
         if POSIX:
-            self.assertEqual(code, signal.SIGTERM)
+            self.assertEqual(code, -signal.SIGTERM)
         else:
             self.assertEqual(code, 0)
         self.assertFalse(p.is_running())
@@ -231,7 +231,7 @@ class TestProcess(unittest.TestCase):
             else:
                 break
         if POSIX:
-            self.assertEqual(code, signal.SIGKILL)
+            self.assertEqual(code, -signal.SIGKILL)
         else:
             self.assertEqual(code, 0)
         self.assertFalse(p.is_running())
@@ -281,6 +281,16 @@ class TestProcess(unittest.TestCase):
         if (max([kernel_time, ktime]) - min([kernel_time, ktime])) > 0.1:
             self.fail("expected: %s, found: %s" % (ktime, kernel_time))
 
+    @unittest.skipUnless(hasattr(psutil.Process, "cpu_num"),
+                         "platform not supported")
+    def test_cpu_num(self):
+        p = psutil.Process()
+        num = p.cpu_num()
+        self.assertGreaterEqual(num, 0)
+        if psutil.cpu_count() == 1:
+            self.assertEqual(num, 0)
+        self.assertIn(p.cpu_num(), range(psutil.cpu_count()))
+
     def test_create_time(self):
         sproc = get_test_subprocess()
         now = time.time()
@@ -501,10 +511,8 @@ class TestProcess(unittest.TestCase):
         try:
             step2 = p.num_threads()
             self.assertEqual(step2, step1 + 1)
-            thread.stop()
         finally:
-            if thread._running:
-                thread.stop()
+            thread.stop()
 
     @unittest.skipUnless(WINDOWS, 'WINDOWS only')
     def test_num_handles(self):
@@ -524,7 +532,6 @@ class TestProcess(unittest.TestCase):
 
         thread = ThreadTask()
         thread.start()
-
         try:
             step2 = p.threads()
             self.assertEqual(len(step2), len(step1) + 1)
@@ -536,11 +543,8 @@ class TestProcess(unittest.TestCase):
             self.assertEqual(athread.id, athread[0])
             self.assertEqual(athread.user_time, athread[1])
             self.assertEqual(athread.system_time, athread[2])
-            # test num threads
-            thread.stop()
         finally:
-            if thread._running:
-                thread.stop()
+            thread.stop()
 
     @retry_before_failing()
     # see: https://travis-ci.org/giampaolo/psutil/jobs/111842553
@@ -845,10 +849,13 @@ class TestProcess(unittest.TestCase):
     def test_cpu_affinity(self):
         p = psutil.Process()
         initial = p.cpu_affinity()
+        assert initial, initial
         self.addCleanup(p.cpu_affinity, initial)
+
         if hasattr(os, "sched_getaffinity"):
             self.assertEqual(initial, list(os.sched_getaffinity(p.pid)))
         self.assertEqual(len(initial), len(set(initial)))
+
         all_cpus = list(range(len(psutil.cpu_percent(percpu=True))))
         # setting on travis doesn't seem to work (always return all
         # CPUs on get):
@@ -859,9 +866,18 @@ class TestProcess(unittest.TestCase):
             if hasattr(os, "sched_getaffinity"):
                 self.assertEqual(p.cpu_affinity(),
                                  list(os.sched_getaffinity(p.pid)))
-        #
-        p.cpu_affinity(all_cpus)
-        self.assertEqual(p.cpu_affinity(), all_cpus)
+            # also test num_cpu()
+            if hasattr(p, "num_cpu"):
+                self.assertEqual(p.cpu_affinity()[0], p.num_cpu())
+
+        # [] is an alias for "all eligible CPUs"; on Linux this may
+        # not be equal to all available CPUs, see:
+        # https://github.com/giampaolo/psutil/issues/956
+        p.cpu_affinity([])
+        if LINUX:
+            self.assertEqual(p.cpu_affinity(), p._proc._get_eligible_cpus())
+        else:
+            self.assertEqual(p.cpu_affinity(), all_cpus)
         if hasattr(os, "sched_getaffinity"):
             self.assertEqual(p.cpu_affinity(),
                              list(os.sched_getaffinity(p.pid)))
@@ -1143,7 +1159,8 @@ class TestProcess(unittest.TestCase):
         for p in psutil.process_iter():
             if p.pid == sproc.pid:
                 continue
-            self.assertNotEqual(p.ppid(), this_parent)
+            # XXX: sometimes this fails on Windows; not sure why.
+            self.assertNotEqual(p.ppid(), this_parent, msg=p)
 
     def test_children(self):
         p = psutil.Process()
@@ -1729,6 +1746,12 @@ class TestFetchAllProcesses(unittest.TestCase):
         self.assertTrue(ret.user >= 0)
         self.assertTrue(ret.system >= 0)
 
+    def cpu_num(self, ret, proc):
+        self.assertGreaterEqual(ret, 0)
+        if psutil.cpu_count() == 1:
+            self.assertEqual(ret, 0)
+        self.assertIn(ret, range(psutil.cpu_count()))
+
     def memory_info(self, ret, proc):
         for name in ret._fields:
             self.assertGreaterEqual(getattr(ret, name), 0)
@@ -1785,8 +1808,11 @@ class TestFetchAllProcesses(unittest.TestCase):
             try:
                 st = os.stat(ret)
             except OSError as err:
+                if WINDOWS and err.errno in \
+                        psutil._psplatform.ACCESS_DENIED_SET:
+                    pass
                 # directory has been removed in mean time
-                if err.errno != errno.ENOENT:
+                elif err.errno != errno.ENOENT:
                     raise
             else:
                 self.assertTrue(stat.S_ISDIR(st.st_mode))
@@ -1799,6 +1825,9 @@ class TestFetchAllProcesses(unittest.TestCase):
 
     def cpu_affinity(self, ret, proc):
         assert ret != [], ret
+        cpus = range(psutil.cpu_count())
+        for n in ret:
+            self.assertIn(n, cpus)
 
     def terminal(self, ret, proc):
         if ret is not None:
